@@ -17,16 +17,10 @@
  * along with QCAD.
  */
 
+include("scripts/library.js");
 include("scripts/WidgetFactory.js");
 include("scripts/Widgets/OptionsToolBar/OptionsToolBar.js");
 
-if (RSettings.isGuiEnabled()) {
-    if (exists("scripts/Widgets/CadToolBar/CadToolBar.js")) {
-        include("scripts/Widgets/CadToolBar/ColumnLayout.js");
-        include("scripts/Widgets/CadToolBar/CadToolBar.js");
-    }
-}
-
 /**
  * Base class for all ECMAScript based actions.
  */
@@ -153,10 +147,10 @@ EAction.prototype.finishEvent = function() {
 
     // reset CAD tool bar:
     if (!isNull(guiAction) && guiAction.getGroup() === "") {
-        if (typeof(CadToolBar) !== "undefined") {
-            if (CadToolBar.getCurrentPanelName()==="SnapToolsPanel") {
-                CadToolBar.back();
-            }
+        var appWin = EAction.getMainWindow();
+        var cadToolBar = appWin.findChild("CadToolBar");
+        if (!isNull(cadToolBar) && cadToolBar.getCurrentPanelName()==="SnapToolsPanel") {
+            cadToolBar.back();
         }
     }
 
@@ -393,7 +387,7 @@ EAction.prototype.showUiOptions = function(resume, restoreFromSettings) {
         if (optionsToolBar.floating) {
             optionsToolBar.resize(optionsToolBar.sizeHint.width(), optionsToolBar.height);
         }
-        wOptions.destroy();
+        destr(wOptions);
 
         // automatically add separator to toolbar:
         var a = optionsToolBar.addSeparator();
@@ -430,6 +424,10 @@ EAction.prototype.initUiOptions = function(resume, optionsToolBar) {
     var children = optionsToolBar.children();
     for (var i=0; i<children.length; i++) {
         var child = children[i];
+        if (isNull(child)) {
+            continue;
+        }
+
         var shortCut = child.shortcut;
         if (!isNull(shortCut) && !shortCut.isEmpty()) {
             var str = shortCut.toString();
@@ -498,7 +496,7 @@ EAction.prototype.hideUiOptions = function(saveToSettings) {
         if (!isNull(optionsToolBar)) {
             optionsToolBar.removeAction(a);
         }
-        a.destroy();
+        destr(a);
     }
 
     // delete additional toolbars of this tool if available:
@@ -508,7 +506,7 @@ EAction.prototype.hideUiOptions = function(saveToSettings) {
             if (isDeleted(tb)) {
                 continue;
             }
-            tb.destroy();
+            destr(tb);
         }
     }
 
@@ -545,7 +543,7 @@ EAction.prototype.hideOptionsToolBarWidgets = function(widgets, noSyncWidgets) {
     for (i = 0; i < children.length; ++i) {
         c = children[i];
         if (c["HideInDialogMode"]===true) {
-            c.destroy();
+            destr(c);
         }
         else if (c["MoveToDialog"]===true) {
             widgets.push(c);
@@ -665,7 +663,7 @@ EAction.prototype.showDialog = function() {
         noSyncWidgets[i].setProperty("Loaded", false);
     }
 
-    this.dialog.destroy();
+    destr(this.dialog);
     this.dialog = undefined;
 
 //    var view = EAction.getGraphicsView();
@@ -979,9 +977,10 @@ EAction.getMenu = function(title, objectName, initFunction) {
     var appWin = EAction.getMainWindow();
     var menuBar = appWin.menuBar();
     var menu;
-    for (var i in menuBar.children()) {
-        menu = menuBar.children()[i];
-        if (menu.objectName === objectName) {
+    var children = menuBar.children();
+    for (var i in children) {
+        menu = children[i];
+        if (!isNull(menu) && menu.objectName === objectName) {
             break;
         }
         menu = undefined;
@@ -1091,13 +1090,13 @@ EAction.getToolBar = function(title, objectName, toolBarArea, category, before)
         return undefined;
     }
 
-    var tb;
+    var tb = undefined;
     if (objectName.length !== 0) {
         tb = appWin.findChild(objectName);
     }
 
     if (!isNull(tb) && !isOfType(tb, QToolBar)) {
-        qWarning("Not a toolbar: ", tb);
+        qWarning("Not a toolbar: " + tb);
         return undefined;
     }
 
@@ -1152,7 +1151,7 @@ EAction.getOptionsToolBar = function() {
     }
 
     if (!isNull(EAction.optionsToolBar)) {
-        if (!QCoreApplication.arguments().contains("-no-show")) {
+        if (!RSettings.getOriginalArguments().contains("-no-show")) {
             EAction.optionsToolBar.visible = true;
         }
     }
@@ -1192,46 +1191,37 @@ EAction.getMainCadToolBarPanel = function() {
  * back to the main CAD toolbar panel.
  */
 EAction.getCadToolBarPanel = function(title, objectName, hasBackButton) {
-    if (typeof(CadToolBar) === "undefined") {
-        return undefined;
-    }
-
     var appWin = EAction.getMainWindow();
-    var cadToolBar = appWin.findChild("CadQToolBar");
+    var cadToolBar = appWin.findChild("CadToolBar");
 
     // create CAD toolbar if it does not exist already:
     if (isNull(cadToolBar)) {
-        var toolBar = new QToolBar(qsTr("CAD Tools"), appWin);
-        toolBar.objectName = "CadQToolBar";
-        cadToolBar = new CadToolBar(toolBar);
-        toolBar.addWidget(cadToolBar);
-        cadToolBar.updateIconSize();
+        cadToolBar = new RCadToolBar(qsTr("CAD Tools"), appWin);
+        cadToolBar.objectName = "CadToolBar";
+//        cadToolBar.updateIconSize();
 
-        toolBar.topLevelChanged.connect(function(onOff) {
+        cadToolBar.topLevelChanged.connect(function(onOff) {
             RSettings.setValue("CadToolBar/VerticalWhenFloating", false);
         });
-        toolBar.orientationChanged.connect(function(orientation) {
+        cadToolBar.orientationChanged.connect(function(orientation) {
             RSettings.setValue("CadToolBar/VerticalWhenFloating", false);
         });
     }
 
-    return CadToolBar.getPanel(title, objectName, hasBackButton);
+    return cadToolBar.getPanel(title, objectName, hasBackButton);
 };
 
 /**
  * Shows the CAD toolbar panel with the given objectName.
  */
 EAction.showCadToolBarPanel = function(objectName) {
-    if (typeof (CadToolBar) == "undefined") {
-        return;
-    }
-
     var appWin = EAction.getMainWindow();
     if (isNull(appWin)) {
         return;
     }
 
-    CadToolBar.showPanel(objectName);
+    var toolBar = appWin.findChild("CadToolBar");
+    toolBar.showPanel(objectName);
 };
 
 /**
@@ -1395,8 +1385,6 @@ EAction.addGuiActionTo = function(action, iface, addToMenu, addToToolBar,
         qWarning("EAction.js:", "addGuiActionTo(): iface not defined");
     }
 
-
-
     if (action.icon.isNull() && !action.isIconDisabled()) {
         action.setIcon(autoPath("scripts/Empty.svg"));
     }
@@ -1752,7 +1740,8 @@ EAction.prototype.applyOperation = function() {
         return false;
     }
     this.complementOperation(op);
-    di.applyOperation(op);
+    var t = di.applyOperation(op);
+    this.postOperation(t);
     return true;
 };
 
@@ -1763,6 +1752,13 @@ EAction.prototype.complementOperation = function(op) {
     return;
 };
 
+/**
+ * Can be reimplemented to work with the transaction returned by the operation.
+ */
+EAction.prototype.postOperation = function(transaction) {
+    return;
+};
+
 /**
  * High level event triggered when a coordinate was specified by the user of a
  * coordinate specification should be previewed.
@@ -2202,8 +2198,8 @@ EAction.getEntityId = function(di, action, event, preview, selectable) {
         }
 
         reactor = new Reactor(id);
-        a.triggered.connect(reactor, "trigger");
-        a.hovered.connect(reactor, "hover");
+        a.triggered.connect(reactor, reactor.trigger);
+        a.hovered.connect(reactor, reactor.hover);
     }
 
     // show context menu:
